"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dag_1 = require("./dag/dag");
const cids_1 = __importDefault(require("cids"));
const tupelo_1 = require("../tupelo");
const transactions_pb_1 = require("tupelo-messages/transactions/transactions_pb");
const dagCBOR = require('ipld-dag-cbor');
/**
 * ChainTree is the main class used for interacting with the data of Tupelo.
 * See {@link https://docs.quorumcontrol.com/docs/chaintree.html} for a detailed description
 * of what a ChainTree is.
 * @public
 */
class ChainTree extends dag_1.Dag {
    /**
     * Creates a new ChainTree
     * @param opts - {@link IChainTreeInitializer}
     * @public
     */
    constructor(opts) {
        super(opts.tip, opts.store);
        this.key = opts.key;
        this.store = opts.store;
    }
    /**
     * resolveData is the mirror image of setData. It starts at the data section
     * of a ChainTree. This allows you to ignore the "tree/data" part of a path (as is done
     * in setData)
     * @param path - the path (starting after /tree/data) you want to resolve
     */
    resolveData(path) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.resolve("/tree/data/" + path);
        });
    }
    /**
     * Returns the DID of the ChainTree
     * @public
     */
    id() {
        return __awaiter(this, void 0, void 0, function* () {
            const resolveResp = yield this.resolve("id");
            return resolveResp.value;
        });
    }
}
exports.ChainTree = ChainTree;
/**
 * Creates a new empty chaintree using the specified key and blockservice.
 * @param store - The {@link IBlockService} to store the new blocks in (Community exports a block service)
 * @param key - The {@link EcdsaKey} to use to name the ChainTree (this is used to create the DID)
 * @public
 */
ChainTree.newEmptyTree = (store, key) => __awaiter(void 0, void 0, void 0, function* () {
    const tip = yield tupelo_1.Tupelo.newEmptyTree(store, key.publicKey);
    return new ChainTree({
        key: key,
        tip: tip,
        store: store,
    });
});
/**
 * getLatest returns a ChainTree with the newest state already cached in the local repo
 * @param did - the did of the tree
 * @public
 */
ChainTree.getLatest = (did) => __awaiter(void 0, void 0, void 0, function* () {
    return tupelo_1.Tupelo.getLatest(did);
});
const setOwnershipPayload = (newOwnerKeys) => {
    var payload = new transactions_pb_1.SetOwnershipPayload();
    payload.setAuthenticationList(newOwnerKeys);
    return payload;
};
/**
 * returns a setOwnershipTransaction
 * @param newOwnerKeys - An array of the addresses of the new owners
 * @public
 */
exports.setOwnershipTransaction = (newOwnerKeys) => {
    var payload = setOwnershipPayload(newOwnerKeys);
    var txn = new transactions_pb_1.Transaction();
    txn.setType(transactions_pb_1.Transaction.Type.SETOWNERSHIP);
    txn.setSetOwnershipPayload(payload);
    return txn;
};
const setDataPayloadMaker = (path, value) => {
    var cborData = dagCBOR.util.serialize(value);
    var payload = new transactions_pb_1.SetDataPayload();
    payload.setPath(path);
    payload.setValue(cborData);
    return payload;
};
/**
 * Returns a setDataTransaction
 * @param path - The path of the Tree to set
 * @param value - An object to set at the path (this will be CBOR encoded for you).
 * @public
 */
exports.setDataTransaction = (path, value) => {
    var payload = setDataPayloadMaker(path, value);
    var txn = new transactions_pb_1.Transaction();
    txn.setType(transactions_pb_1.Transaction.Type.SETDATA);
    txn.setSetDataPayload(payload);
    return txn;
};
const establishTokenPayload = (name, maximum) => {
    var policy = new transactions_pb_1.TokenMonetaryPolicy();
    policy.setMaximum(maximum);
    var payload = new transactions_pb_1.EstablishTokenPayload();
    payload.setName(name);
    payload.setMonetaryPolicy(policy);
    return payload;
};
/**
 * Returns a new establishTokenTransaction which is used to setup the monetary policy
 * of a new token on a ChainTree
 * @public
 */
exports.establishTokenTransaction = (name, maximum) => {
    var payload = establishTokenPayload(name, maximum);
    var txn = new transactions_pb_1.Transaction();
    txn.setType(transactions_pb_1.Transaction.Type.ESTABLISHTOKEN);
    txn.setEstablishTokenPayload(payload);
    return txn;
};
const mintTokenPayload = (name, amount) => {
    var payload = new transactions_pb_1.MintTokenPayload();
    payload.setName(name);
    payload.setAmount(amount);
    return payload;
};
/**
 * @public
 */
exports.mintTokenTransaction = (name, amount) => {
    var payload = mintTokenPayload(name, amount);
    var txn = new transactions_pb_1.Transaction();
    txn.setType(transactions_pb_1.Transaction.Type.MINTTOKEN);
    txn.setMintTokenPayload(payload);
    return txn;
};
const sendTokenPayload = (sendId, name, amount, destinationChainId) => {
    var payload = new transactions_pb_1.SendTokenPayload();
    payload.setId(sendId);
    payload.setName(name);
    payload.setAmount(amount);
    payload.setDestination(destinationChainId);
    return payload;
};
/**
 * @public
 */
exports.sendTokenTransaction = (sendId, name, amount, destinationChainId) => {
    var payload = sendTokenPayload(sendId, name, amount, destinationChainId);
    var txn = new transactions_pb_1.Transaction();
    txn.setType(transactions_pb_1.Transaction.Type.SENDTOKEN);
    txn.setSendTokenPayload(payload);
    return txn;
};
const receiveTokenPayload = (sendId, tip, proof, leaves) => {
    var payload = new transactions_pb_1.ReceiveTokenPayload();
    payload.setSendTokenTransactionId(sendId);
    payload.setTip(tip);
    payload.setProof(proof);
    payload.setLeavesList(leaves);
    return payload;
};
/**
 * @public
 */
exports.receiveTokenTransaction = (sendId, tip, proof, leaves) => {
    var payload = receiveTokenPayload(sendId, tip, proof, leaves);
    var txn = new transactions_pb_1.Transaction();
    txn.setType(transactions_pb_1.Transaction.Type.RECEIVETOKEN);
    txn.setReceiveTokenPayload(payload);
    return txn;
};
/**
 * This function is useful if you have the full payload from the send token sent to you
 * no need to deconstruct and the reconstruct the payload. Often wallets will send this payload as base64.
 *
 * @param payload - The receive token payload
 * @public
 */
exports.receiveTokenTransactionFromPayload = (payload) => {
    var txn = new transactions_pb_1.Transaction();
    txn.setType(transactions_pb_1.Transaction.Type.RECEIVETOKEN);
    const tip = new cids_1.default(payload.getTip());
    const leaves = payload.getLeavesList_asU8();
    if (leaves === undefined) {
        throw new Error('leaves must be defined');
    }
    const proof = payload.getProof();
    if (proof === undefined) {
        throw new Error("proof must be defined");
    }
    txn.setReceiveTokenPayload(receiveTokenPayload(payload.getTransactionId(), tip.buffer, proof, leaves));
    return txn;
};
exports.default = ChainTree;
