/// <reference types="node" />
import CID from 'cids';
/**
 * An IPFS Block
 * @public
 */
export interface IBlock {
    data: Buffer;
    cid: CID;
}
/**
 * An IPFS Bitswap instance.
 * @public
 */
export interface IBitSwap {
    get(cid: CID, callback: Function): void;
    put(block: IBlock, callback: Function): void;
    start(callback: Function): void;
    stop(callback: Function): void;
}
/**
 * The interface to a js-ipfs block-service. See: {@link https://ipfs.github.io/js-ipfs-block-service}
 * @public
 */
export interface IBlockService {
    put(block: IBlock): Promise<any>;
    get(cid: CID): Promise<IBlock>;
    delete(cid: CID): Promise<any>;
    setExchange(bitswap: IBitSwap): void;
    unsetExchange(): void;
    hasExchange(): boolean;
}
interface IDagStoreResolveResponse {
    remainderPath: string;
    value: any;
}
interface IExtendedDagStoreIterator {
    first(): Promise<IDagStoreResolveResponse>;
    last(): Promise<IDagStoreResolveResponse>;
    all(): Promise<IDagStoreResolveResponse[]>;
}
/**
 * An IPFS DagStore instance
 * @public
 */
export interface IDagStore {
    get(cid: CID): Promise<Object>;
    resolve(cid: CID, path: string): IExtendedDagStoreIterator;
}
/**
 * This defines the resolve response when doing queries against the DAG from IPFS
 * @public
 */
export interface IResolveResponse {
    remainderPath: string[];
    value: any;
}
/**
 * Underlies a ChainTree, it represents a DAG of IPLD nodes and supports resolving accross
 * multiple nodes.
 * @public
 */
export declare class Dag {
    tip: CID;
    dagStore: IDagStore;
    constructor(tip: CID, store: IBlockService);
    /**
     * Gets a node from the dag
     * @param cid - The CID of the node to get from the DAG
     * @public
     */
    get(cid: CID): Promise<Object>;
    /**
     *
     * @param path - a path to the desired node/key in the DAG (eg /path/to/data). Array form (eg ['path', 'to', 'data']) is deprecated
     * @public
     */
    resolve(path: Array<string> | string): Promise<IResolveResponse>;
    /**
     * Similar to resolve, but allows you to start at a specific tip of a dag rather than the current tip.
     * @param tip - The tip of the dag to start at
     * @param path - the path to find the value. Array form is deprecated, use string form (eg /path/to/data) instead
     * @public
     */
    resolveAt(tip: CID, path: Array<string> | string): Promise<IResolveResponse>;
}
export {};
//# sourceMappingURL=dag.d.ts.map