/// <reference types="node" />
import EventEmitter from 'events';
import { IP2PNode } from '../node';
import { NotaryGroup } from 'tupelo-messages';
import { Transaction } from 'tupelo-messages/transactions/transactions_pb';
import CID from 'cids';
import { IBlockService } from '../chaintree/dag/dag';
import { ChainTree } from '../chaintree';
import Repo from '../repo';
interface IRepo {
    blocks: IBlockService;
}
/**
 * Commmunity is a combination of a Tupelo NotaryGroup and the application 'Community' running.
 * The remote p2p community app will also store blocks, and this is the default supported way
 * that the current wasm-sdk works.
 * @public
 */
export declare class Community extends EventEmitter {
    node: IP2PNode;
    group: NotaryGroup;
    private repo;
    bitswap: any;
    blockservice: IBlockService;
    private _started;
    private _startPromise;
    private _startPromiseResolve;
    private _startPromiseReject;
    constructor(node: IP2PNode, group: NotaryGroup, repo: IRepo);
    waitForStart(): Promise<Community>;
    /**
     * getProof returns the proof for the current tip of a ChainTree
     * for a given ChainTree (its DID)
     * @param did - The DID of the ChainTree
     * @public
    */
    getProof(did: string): Promise<import("tupelo-messages/gossip/gossip_pb").Proof>;
    /**
     * returns the TIP as a CID of the ChainTree. This is more of a convenience function
     * around getting the current state, and then casting the tip, etc.
     * @param did - The DID of the ChainTree
     */
    getTip(did: string): Promise<CID>;
    sendTokenAndGetPayload(tree: ChainTree, tx: Transaction): Promise<import("tupelo-messages").TokenPayload>;
    /**
     * playTransactions is a convenience wrapper on community to make calling the underlying Tupelo.playTransactions
     * easier when using a fully community client
    */
    playTransactions(tree: ChainTree, transactions: Transaction[]): Promise<import("tupelo-messages/gossip/gossip_pb").Proof>;
    /** next update is a helper function
     * which lets you do an await until the next tip
     * update of the community
     * @deprecated
    */
    nextUpdate(): Promise<void>;
    /**
     * starts up the community
    */
    start(): Promise<Community>;
    stop(): Promise<void>;
}
/**
 * This waits until the libp2p node has connected to at least 2/3 of signers
 * @private
 */
export declare function afterQuorumSignersConnected(node: IP2PNode, group: NotaryGroup): Promise<void>;
export declare namespace Community {
    /**
     * getDefault returns the default Tupelo commuity. This is currently the Tupelo testnet.
     * It creates a libp2p node and connects to the testnet, establishes the community connection.
     * This method optionally takes a {@link Repo}. If you do not pass in a Repo, then it will create a
     * default repo. Details on the default can be found:
     * @param repo - (optional) - a {@link Repo} object (wrapper around an IPFS repo).
     * @public
    */
    function getDefault(repo?: Repo): Promise<Community>;
    /**
     * setDefault allows you to set a community you control so that when code calls Community.getDefault() it
     * returns this community. This is useful in situations like local testing, where your test harness can
     * point the code at a local community.
     * @param community - the {@link Community} to set as default
     * @public
     */
    function setDefault(community: Community): void;
    /**
     * fromNotaryGroup creates a community from a notary group TOML string without requiring boiler plate code for
     * creating a new libp2p node. repo argument is optional and will create one if not specified.
     * @param tomlString - the TOML for the notary group
     * @param repo - (optional) the repo to use for this notary group. Will default to an ondisk repo named after the notary group
     */
    function fromNotaryGroupToml(tomlString: string, repo?: Repo): Promise<Community>;
    /**
     * fromNotaryGroup creates a community from a NotaryGroup without requiring boiler plate code for
     * creating a new libp2p node. repo argument is optional and will create one if not specified.
     * @param notaryGroup - the notaryGroup
     * @param repo - (optional) the repo to use for this notary group. Will default to an ondisk repo named after the notary group
     */
    function fromNotaryGroup(notaryGroup: NotaryGroup, repo?: Repo): Promise<Community>;
}
export {};
//# sourceMappingURL=community.d.ts.map