"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = __importDefault(require("events"));
const node_1 = require("../node");
const transactions_pb_1 = require("tupelo-messages/transactions/transactions_pb");
const cids_1 = __importDefault(require("cids"));
const wrappedblockservice_1 = require("./wrappedblockservice");
const tupelo_1 = __importDefault(require("../tupelo"));
const default_1 = require("./default");
const debug_1 = __importDefault(require("debug"));
const repo_1 = __importDefault(require("../repo"));
const notarygroup_1 = __importStar(require("../notarygroup"));
const debugLog = debug_1.default("community");
// const IpfsBlockService:any = require('ipfs-block-service');
const IpfsBitswap = require('ipfs-bitswap');
const IpfsBlockService = require('ipfs-block-service');
/**
 * Commmunity is a combination of a Tupelo NotaryGroup and the application 'Community' running.
 * The remote p2p community app will also store blocks, and this is the default supported way
 * that the current wasm-sdk works.
 * @public
 */
class Community extends events_1.default {
    constructor(node, group, repo) {
        super();
        this._started = false;
        this.node = node;
        this.group = group;
        this.repo = repo;
        this.bitswap = new IpfsBitswap(this.node, this.repo.blocks);
        this.blockservice = new wrappedblockservice_1.WrappedBlockService(new IpfsBlockService(this.repo));
        this.blockservice.setExchange(this.bitswap);
        this._startPromiseResolve = () => { }; // replaced on the line below, this just stops typescript from complaining
        this._startPromiseReject = () => { }; // replaced on the line below, this just stops typescript from complaining
        this._startPromise = new Promise((resolve) => { this._startPromiseResolve = resolve; });
    }
    waitForStart() {
        return this._startPromise;
    }
    /**
     * getProof returns the proof for the current tip of a ChainTree
     * for a given ChainTree (its DID)
     * @param did - The DID of the ChainTree
     * @public
    */
    getProof(did) {
        return tupelo_1.default.getTip(did);
    }
    /**
     * returns the TIP as a CID of the ChainTree. This is more of a convenience function
     * around getting the current state, and then casting the tip, etc.
     * @param did - The DID of the ChainTree
     */
    getTip(did) {
        return __awaiter(this, void 0, void 0, function* () {
            const proof = yield tupelo_1.default.getTip(did);
            return new cids_1.default(Buffer.from(proof.getTip_asU8()));
        });
    }
    sendTokenAndGetPayload(tree, tx) {
        return __awaiter(this, void 0, void 0, function* () {
            const sendTokenPayload = tx.getSendTokenPayload();
            if (tx.getType() != transactions_pb_1.Transaction.Type.SENDTOKEN || sendTokenPayload === undefined) {
                throw new Error("must use a send token transaction here");
            }
            const proof = yield this.playTransactions(tree, [tx]);
            return tupelo_1.default.tokenPayloadForTransaction({
                blockService: this.blockservice,
                tip: tree.tip,
                tokenName: sendTokenPayload.getName(),
                sendId: sendTokenPayload.getId(),
                proof: proof,
            });
        });
    }
    /**
     * playTransactions is a convenience wrapper on community to make calling the underlying Tupelo.playTransactions
     * easier when using a fully community client
    */
    playTransactions(tree, transactions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield tupelo_1.default.playTransactions(tree, transactions);
        });
    }
    /** next update is a helper function
     * which lets you do an await until the next tip
     * update of the community
     * @deprecated
    */
    nextUpdate() {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO: deprecated
            // let resolve: Function
            // const p = new Promise((res, _rej) => { resolve = res })
            // this.once('tip', () => { resolve() })
            return Promise.resolve();
        });
    }
    /**
     * starts up the community
    */
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._started) {
                return this._startPromise;
            }
            this._started = true;
            debugLog("start()");
            yield this.bitswap.start();
            debugLog("bitswap started");
            yield tupelo_1.default.startClient(this.node.pubsub, this.group, this.blockservice);
            debugLog("started");
            this._startPromiseResolve(this);
            this.emit('start');
            return this._startPromise;
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            this.bitswap.stop(() => { });
            this.node.stop();
        });
    }
}
exports.Community = Community;
/**
 * This waits until the libp2p node has connected to at least 2/3 of signers
 * @private
 */
function afterQuorumSignersConnected(node, group) {
    return __awaiter(this, void 0, void 0, function* () {
        const peerIds = yield notarygroup_1.notaryGroupToSignerPeerIds(group);
        const minConnected = Math.ceil((peerIds.length / 3) * 2);
        debugLog(`waiting for ${minConnected} signers to connect, list: `, peerIds.map((id) => { return id.toB58String(); }));
        return new Promise((resolve) => {
            let connected = 0;
            const onConnect = (peer) => {
                debugLog("peer connected: ", peer.id.toB58String());
                for (let id of peerIds) {
                    if (id.isEqual(peer.id)) {
                        connected++;
                        debugLog(`${connected} / ${minConnected} signers connected`);
                        if (connected >= minConnected) {
                            node.off('peer:connect', onConnect);
                            resolve();
                            return;
                        }
                    }
                }
            };
            node.on('peer:connect', onConnect);
        });
    });
}
exports.afterQuorumSignersConnected = afterQuorumSignersConnected;
(function (Community) {
    /**
     * getDefault returns the default Tupelo commuity. This is currently the Tupelo testnet.
     * It creates a libp2p node and connects to the testnet, establishes the community connection.
     * This method optionally takes a {@link Repo}. If you do not pass in a Repo, then it will create a
     * default repo. Details on the default can be found:
     * @param repo - (optional) - a {@link Repo} object (wrapper around an IPFS repo).
     * @public
    */
    function getDefault(repo) {
        return default_1._getDefault(repo);
    }
    Community.getDefault = getDefault;
    /**
     * setDefault allows you to set a community you control so that when code calls Community.getDefault() it
     * returns this community. This is useful in situations like local testing, where your test harness can
     * point the code at a local community.
     * @param community - the {@link Community} to set as default
     * @public
     */
    function setDefault(community) {
        return default_1._setDefault(community);
    }
    Community.setDefault = setDefault;
    /**
     * fromNotaryGroup creates a community from a notary group TOML string without requiring boiler plate code for
     * creating a new libp2p node. repo argument is optional and will create one if not specified.
     * @param tomlString - the TOML for the notary group
     * @param repo - (optional) the repo to use for this notary group. Will default to an ondisk repo named after the notary group
     */
    function fromNotaryGroupToml(tomlString, repo) {
        const ng = notarygroup_1.default(tomlString);
        return fromNotaryGroup(ng, repo);
    }
    Community.fromNotaryGroupToml = fromNotaryGroupToml;
    /**
     * fromNotaryGroup creates a community from a NotaryGroup without requiring boiler plate code for
     * creating a new libp2p node. repo argument is optional and will create one if not specified.
     * @param notaryGroup - the notaryGroup
     * @param repo - (optional) the repo to use for this notary group. Will default to an ondisk repo named after the notary group
     */
    function fromNotaryGroup(notaryGroup, repo) {
        return new Promise((res, rej) => __awaiter(this, void 0, void 0, function* () {
            const node = yield node_1.p2p.createNode({
                namespace: notaryGroup.getId(),
                bootstrapAddresses: notaryGroup.getBootstrapAddressesList()
            });
            if (repo == undefined) {
                repo = new repo_1.default(notaryGroup.getId());
                try {
                    yield repo.init({});
                    yield repo.open();
                }
                catch (e) {
                    rej(e);
                }
            }
            const c = new Community(node, notaryGroup, repo.repo);
            afterQuorumSignersConnected(node, notaryGroup).then(() => __awaiter(this, void 0, void 0, function* () {
                res(yield c.start());
            }));
            node.start(() => __awaiter(this, void 0, void 0, function* () {
                debugLog("p2p node started");
            }));
        }));
    }
    Community.fromNotaryGroup = fromNotaryGroup;
})(Community = exports.Community || (exports.Community = {}));
