"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_pb_1 = require("tupelo-messages/config/config_pb");
const toml_1 = __importDefault(require("toml"));
const PeerId = require('peer-id');
const cryptoKeys = require('libp2p-crypto/src/keys');
const Secp256k1PublicKey = cryptoKeys.supportedKeys.secp256k1.Secp256k1PublicKey;
/**
 * publicKeySetToPeerId returns the libp2p peerID from the signer dstKey config
 * @param set - the PublicKeySet (this is a signer in the NotaryGroup)
 */
function publicKeySetToPeerId(set) {
    return new Promise((resolve, reject) => {
        const key = new Secp256k1PublicKey(Buffer.from(set.getDestKey_asU8()));
        PeerId.createFromPubKey(key.bytes, (err, resp) => {
            if (err) {
                reject(err);
                return;
            }
            resolve(resp);
        });
    });
}
exports.publicKeySetToPeerId = publicKeySetToPeerId;
function notaryGroupToSignerPeerIds(ng) {
    let _idPromises = ng.getSignersList().map((signer) => {
        return publicKeySetToPeerId(signer);
    });
    return Promise.all(_idPromises);
}
exports.notaryGroupToSignerPeerIds = notaryGroupToSignerPeerIds;
function hexToBuffer(hex) {
    // the hex that we export in go is always prefaced with "0x", so strip that off
    return Buffer.from(hex.substr(2), "hex");
}
// stack overflow ftw
function toCamel(o) {
    let newO, origKey, newKey, value;
    if (o instanceof Array) {
        return o.map(function (value) {
            if (typeof value === "object") {
                value = toCamel(value);
            }
            return value;
        });
    }
    else {
        newO = {};
        for (origKey in o) {
            if (Object.prototype.hasOwnProperty.call(o, origKey)) {
                newKey = (origKey.charAt(0).toLowerCase() + origKey.slice(1) || origKey).toString();
                value = o[origKey];
                if (value instanceof Array || (value !== null && value.constructor === Object)) {
                    value = toCamel(value);
                }
                newO[newKey] = value;
            }
        }
    }
    return newO;
}
exports.toCamel = toCamel;
/**
 * takes a toml config and returns a NotaryGroup object.
 * @param tomlString - a notary group HumanConfig in toml format
 * @public
 */
function tomlToNotaryGroup(tomlString) {
    const config = toml_1.default.parse(tomlString);
    return configToNotaryGroup(toCamel(config));
}
exports.tomlToNotaryGroup = tomlToNotaryGroup;
function configToNotaryGroup(obj) {
    const ng = new config_pb_1.NotaryGroup();
    ng.setId(obj.id);
    ng.setBootstrapAddressesList(obj.bootstrapAddresses);
    let signers = [];
    for (var s of obj.signers) {
        let publicKeySet = new config_pb_1.PublicKeySet();
        publicKeySet.setVerKey(hexToBuffer(s.verKeyHex));
        publicKeySet.setDestKey(hexToBuffer(s.destKeyHex));
        signers = signers.concat(publicKeySet);
    }
    ng.setSignersList(signers);
    return ng;
}
exports.configToNotaryGroup = configToNotaryGroup;
exports.default = tomlToNotaryGroup;
